#ifndef _F2D_FFMPEG2DIRAC_H
#define _F2D_FFMPEG2DIRAC_H
#include <stdlib.h>

typedef struct ff2dirac_metadata {
    char *key;
    char *value;
    struct ff2dirac_metadata *next;
} ff2dirac_metadata;

typedef struct ff2dirac_subtitle{
    char *text;
    size_t len;
    double t0;
    double t1;
#ifdef HAVE_KATE
    kate_region kr;
    kate_palette kp;
    kate_bitmap kb;
#endif
} ff2dirac_subtitle;

typedef struct ff2dirac_kate_stream{
    /* this block valid for subtitles loaded from a file */
    const char *filename;
    size_t num_subtitles;
    ff2dirac_subtitle *subtitles;

    /* this block valid for subtitles coming from the source video */
    int stream_index;

    /* this block valid for all subtitle sources */
    size_t subtitles_count; /* total subtitles output so far */
    char *subtitles_encoding;
    char subtitles_language[16];
    char subtitles_category[16];
} ff2dirac_kate_stream;

typedef enum {
    VCODEC_DIRAC,
    VCODEC_SCHRO,
    VCODEC_THEORA
} vcodec_enum_t;

typedef struct ff2dirac{
    AVFormatContext *context;
    int video_index;
    int audio_index;

    int deinterlace;
    int soft_target;
    int buf_delay;
    int vhook;
    int disable_video;
    int no_upscaling;

    int audiostream;
    int sample_rate;
    int channels;
    int disable_audio;
    float audio_quality;
    int audio_bitrate;
    int preset;

    int included_subtitles;
    int disable_metadata;
    int disable_oshash;

    vcodec_enum_t vcodec_type;
    int videostream;
    int picture_width;
    int picture_height;
    double fps;
    struct SwsContext *sws_colorspace_ctx; /* for image resampling/resizing */
    struct SwsContext *sws_scale_ctx; /* for image resampling/resizing */
    ReSampleContext *audio_resample_ctx;
    ogg_int32_t aspect_numerator;
    ogg_int32_t aspect_denominator;
    int color_primaries;
    AVRational pixel_aspect;
    AVRational frame_aspect;
    int max_x;
    int max_y;
    int speed_level;
    int full_search;
    int pix_fmt;
    double video_quality;
    int video_bitrate;
    ogg_uint32_t keyint;  /* Theora only */
    char pp_mode[255];
#if defined(HAVE_DIRAC) || defined (HAVE_SCHROEDINGER)
    int lossless;           /* code in lossless mode */
    char *prefilter;         /* "NONE", "CWM", "RECTLP",  "DIAGLP" */
    int prefilter_strength; /* Amount of prefiltering. Higher the value the more
                               the input is prefiltered */
    int intra_only;         /* Only intra pictures and no inter pictures */
    int numL1;              /* Number of backward predicted frames per GOP. If
                               set to 0, the sequence will be coded as intra
                               only*/
    int sepL1;              /* Number of bi-directional predicted frames between
                               bacward predicted frames in a GOP */
    char * mv_prec;         /* Motion vector precision "1", "1/2", "1/4"
                               "1/8" */
    int mc_block_width;     /* Width of motion compensated blocks. Must be multiples of 4 */
    int mc_block_height;    /* Height of motion compensated blocks. Must be multiples of 4 */
    int mc_block_overlap;   /* % Motion compensated block overlap */
    int combined_me;        /* Use all three components for motion estimation */
    char *dwt_intra;         /* Wavelet filter to use for Intra pictures */
    char *dwt_inter;         /* Wavelet filter to use for Inter pictures */
                            /* Valid values "DD9_7", "LEGALL5_3", "DD13_7",
                               "HAAR0", "HAAR1", "FIDELITY", "DAUB9_7"*/
    int dwt_depth;          /* Number of Wavelet levels */
    int multi_quants;        /* Use multiple quantisers per sub-band */
    int no_spartition;      /* Disable spatial partitioning. i.e. only one
                               code-block per sub-band. */
    int no_ac;              /* Use VLC instead of arithmetic coding for entropy
                              coding. Only valid in intra only mode*/
    float cpd;              /* Cycles per degree option for visual modelling */
#endif
    AVRational force_input_fps;
    int sync;

    /* cropping */
    int frame_topBand;
    int frame_bottomBand;
    int frame_leftBand;
    int frame_rightBand;

    int frame_width;
    int frame_height;
    int frame_x_offset;
    int frame_y_offset;

    /* In seconds */
    double start_time;
    double end_time;

    AVRational framerate;
    int framerate_new;

    /* Meta data */
    ff2dirac_metadata *metadata;

    int64_t pts_offset_frame; /* frame, which pts is used as pts_offset */
    int64_t pts_offset; /* base value for input pts */
    int64_t frame_count; /* total video frames output so far */
    int64_t sample_count; /* total audio samples output so far */

    size_t n_kate_streams;
    ff2dirac_kate_stream *kate_streams;

    int ignore_non_utf8;
    // ffmpeg2theora --nosound -f dv -H 32000 -S 0 -v 8 -x 384 -y 288 -G 1.5 input.dv
    double video_gamma;
    double video_bright;
    double video_contr;
    double video_satur;
    int y_lut_used;
    int uv_lut_used;
    unsigned char y_lut[256];
    unsigned char uv_lut[256];

    int verbose;
}
*ff2dirac;

#endif
